<?php
/*
Plugin Name: Countdown widget
Description: Adds a sidebar widget to show the amount of time till something happens. Yeah, it's lame.
Author: James Wilson
Version: 1.0
Author URI: http://nothingoutoftheordinary.com
*/

// Takes the time to or from some time, and parses it into a x year(s), x hour(s), x minute(s), x second(s). If it's not enough to be a year, it won't show years, same with hours and minutes
function widget_countdown_timeparse($till) {
	// Time for math!!!
	// This actually took more thinking than I thought it would. I could've probably just looked it up, but where's the fun in that?
	if($till < 60) {
		$format = $till . "second" . ( ($till == 1) ? "" : "s") ;
	}
	elseif($till < 3600) {
		$minutes = floor( $till / 60 );
		$seconds = $till % 60;
		$format = $minutes . " minute" . ( ($minutes == 1) ? "" : "s" ) . " " . $seconds . " second" . ( ($seconds == 1) ? "" : "s");
	}
	elseif($till < 86400) {
		$hours = floor( $till / 3600);
		$minutes = floor( $till / 60 ) % 60;
		$seconds = $till % 3600 % 60;
		$format = $hours . " hour" . ( ($hours == 1) ? "" : "s" ) . " " . $minutes . " minute" . ( ($minutes == 1) ? "" : "s" ) . " " . $seconds . " second" . ( ($seconds == 1) ? "" : "s" );
	}
	elseif($till < 31536000) {
		$days = floor( $till / 86400 );
		$hours = floor( $till / 3600 ) % 24;
		$minutes = floor ( $till / 60 ) % 60;
		$seconds = $till % 86400 % 60;
		$format = $days . " day" . ( ($days == 1) ? "" : "s" ) . " " . $hours . " hour" . ( ($hours == 1) ? "" : "s" ) . " " . $minutes . " minute" . ( ($minutes == 1) ? "" : "s" ) . " " . $seconds . " second" . ( ($seconds == 1) ? "" : "s" );
	}
	else {
		$years = floor( $till / 31536000 );
		$days = floor( $till / 86400 ) % 365;
		$hours = floor( $till / 3600 ) % 24;
		$minutes = floor( $till / 60 ) % 60;
		$seconds = $till % 31536000 % 60;
		$format = $years . " year" . ( ($years == 1) ? "" : "s" ) . " " . $days . " day" . ( ($days == 1) ? "" : "s" ) . " " . $hours . " hour" . ( ($hours == 1) ? "" : "s" ) . " " . $minutes . " minute" . ( ($minutes == 1) ? "" : "s" ) . " " . $seconds . " second" . ( ($seconds == 1) ? "" : "s" );
	}
	
	return $format;

}


// _Very_ largely based on the gsearch widget by Automattic

// Put functions into one big function we'll call at the plugins_loaded
// action. This ensures that all required plugin functions are defined.
function widget_countdown_init() {

	// Check for the required plugin functions. This will prevent fatal
	// errors occurring when you deactivate the dynamic-sidebar plugin.
	if ( !function_exists('register_sidebar_widget') )
		return;

	// This is the function that outputs our little Google search form.
	function widget_countdown($args) {
		
		// $args is an array of strings that help widgets to conform to
		// the active theme: before_widget, before_title, after_widget,
		// and after_title are the array keys. Default tags: li and h2.
		extract($args);

		// Each widget can store its own options. We keep strings here.
		$options = get_option('widget_countdown');
		$title = ( $options['link'] != "" ) ? '<a href="' . $options['link'] . '">'. $options['name'] . '</a>' : $options['name'];
		$time = $options['time'];
		$link = $options['link'];
		$current = time();
		$widgettitle = ($options['title'] != "") ? $before_title.$options['title'].$after_title : "";  // ($options['title'] != "") ? $options['title'] : "countdown Rankings";
		
		if($current > $time) {
			$since = $current - $time;
			$content = $title . " happened " . widget_countdown_timeparse($since) . " ago";
		}
		else {
			$till = $time - $current;
			$content = widget_countdown_timeparse($till) . ' until ' . $title;
		}
		
		echo $before_widget;
		echo $widgettitle;
		echo $content;
		echo $after_widget;
		
	}

	// This is the function that outputs the form to let the users edit
	// the widget's title. It's an optional feature that users cry for.
	function widget_countdown_control() {

		// Get our options and see if we're handling a form submission.
		$options = get_option('widget_countdown');
		if ( !is_array($options) )
			$options = array('title'=>'', 
				'time' => 1211864400,
				'month' => 'May',
				'day' => '27',
				'year' => '2008',
				'hours' => '12:00 AM',
				'name' => 'My Birthday',
				'link' => 'http://nothingoutoftheordinary.com'
			);
		if ( $_POST['countdown-submit'] ) {

			// Remember to sanitize and format use input appropriately.
			$options['title'] = strip_tags(stripslashes($_POST['countdown-title']));
			$options['name'] = strip_tags(stripslashes($_POST['countdown-name']));
			$options['month'] = strip_tags(stripslashes($_POST['countdown-month']));
			$options['day'] = strip_tags(stripslashes($_POST['countdown-day']));
			$options['year'] = strip_tags(stripslashes($_POST['countdown-year']));
			$options['hours'] = strip_tags(stripslashes($_POST['countdown-hours']));
			$options['link'] = strip_tags(stripslashes($_POST['countdown-link']));
			$options['time'] = strtotime($options['month'] . " " . $options['day'] . " " . $options['year'] . " " . $options['hours']);
			update_option('widget_countdown', $options);
		}

		// Be sure you format your options to be valid HTML attributes.
		$title = htmlspecialchars($options['title'], ENT_QUOTES);
		$name = htmlspecialchars($options['name'], ENT_QUOTES);
		$link = htmlspecialchars($options['link'], ENT_QUOTES);
		$month = htmlspecialchars($options['month'], ENT_QUOTES);
		$day = htmlspecialchars($options['day'], ENT_QUOTES);
		$year = htmlspecialchars($options['year'], ENT_QUOTES);
		$hours = htmlspecialchars($options['hours'], ENT_QUOTES);
		
		
		// Here is our little form segment. Notice that we don't need a
		// complete form. This will be embedded into the existing form.
		echo '<p style="text-align:right;"><label for="countdown-title">' . __('Title:') . ' <input style="width: 200px;" id="countdown-title" name="countdown-title" type="text" value="'.$title.'" /></label></p>';
		echo '<p style="text-align:right;"><label for="countdown-name">' . __('Event name:', 'widgets') . ' <input style="width: 200px;" id="countdown-name" name="countdown-name" type="text" value="'.$name.'" /></label></p>';
		echo '<p style="text-align:right;"><label for="countdown-link">' . __('Event Link (optional):', 'widgets') . ' <input style="width: 200px;" id="countdown-link" name="countdown-link" type="text" value="'.$link.'" /></label></p>';
		echo '<p style="text-align:right;"><label for="countdown-month">' . __('Month:', 'widgets') . ' <input style="width: 200px;" id="countdown-month" name="countdown-month" type="text" value="'.$month.'" /></label></p>';
		echo '<p style="text-align:right;"><label for="countdown-day">' . __('Day:', 'widgets') . ' <input style="width: 200px;" id="countdown-day" name="countdown-day" type="text" value="'.$day.'" /></label></p>';
		echo '<p style="text-align:right;"><label for="countdown-year">' . __('Year:', 'widgets') . ' <input style="width: 200px;" id="countdown-year" name="countdown-year" type="text" value="'.$year.'" /></label></p>';
		echo '<p style="text-align:right;"><label for="countdown-hours">' . __('Time:', 'widgets') . ' <input style="width: 200px;" id="countdown-hours" name="countdown-hours" type="text" value="'.$hours.'" /></label></p>';
		echo '<input type="hidden" id="countdown-submit" name="countdown-submit" value="1" />';
	}
	
	// This registers our widget so it appears with the other available
	// widgets and can be dragged and dropped into any active sidebars.
	register_sidebar_widget(array('Countdown', 'widgets'), 'widget_countdown');

	// This registers our optional widget control form. Because of this
	// our widget will have a button that reveals a 300x100 pixel form.
	register_widget_control(array('Countdown', 'widgets'), 'widget_countdown_control', 350, 250);
}

// Run our code later in case this loads prior to any required plugins.
add_action('widgets_init', 'widget_countdown_init');

?>